{% load bootstrap %}
{% load i18n %}

<div class="col-md-8 col-md-pull-4">
    <div class="panel panel-default">
        <div class="panel-heading">
            <div class="panel-title">
                {% translate "Your SRP Request" %}
            </div>
        </div>

        <div class="panel-body">
            <form autocomplete="off" action="{% url 'aasrp:request_srp' srp_link.srp_code %}" method="post">
                {% csrf_token %}

                <fieldset class="aa-srp-request-form">
                    {{ form|bootstrap }}

                    {% include "aasrp/partials/form/required_field_hint.html" %}

                    <div class="form-group aasrp-form-group text-right clearfix">
                        <button class="btn btn-primary" type="submit">
                            {% translate "Submit SRP Request" %}
                        </button>
                    </div>
                </fieldset>
            </form>
        </div>
    </div>
</div>
