{% extends "aasrp/base.html" %}

{% load i18n %}

{% block page_title %}{% translate "SRP Link Edit" %}{% endblock %}

{% block aasrp_header %}
    {% translate "Add/Edit After Action Report Link" as translated_header %}
    {% include "aasrp/partials/header/h1.html" with header_text=translated_header %}
{% endblock %}

{% block aasrp_body %}
    {% include "aasrp/partials/link_edit/form.html" %}
{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_css %}
    {% include "aasrp/bundles/aa-bootstrap-fix-css.html" %}
    {% include "aasrp/bundles/aa-srp-css.html" %}
    {% include "aasrp/bundles/aa-srp-form-css.html" %}
{% endblock %}
