@media all {
    .template-light-mode .aa-srp .nav-tabs > li.active > a {
        background-color: rgb(236 240 241);
        color: rgb(0 0 0);
    }

    .template-dark-mode .aa-srp .nav-tabs > li.active > a {
        background-color: rgb(70 69 69);
        color: rgb(255 255 255);
    }

    .aa-srp .btn-tabs {
        border-width: 1px;
    }

    .btn-icon-aasrp {
        height: 36px;
        margin: 0.1rem;
        width: 36px;
    }

    .btn-icon-aasrp-status {
        cursor: default;
    }

    .btn-icon-aasrp-status:focus {
        border: inherit;
    }

    .srp-payout-amount {
        display: inline-block;
    }

    .srp-payout-amount-changed {
        font-weight: bold;
    }

    .srp-link-actions,
    .srp-request-actions {
        white-space: nowrap;
    }

    .srp-dashboard-user-isk-cost,
    .srp-dashboard-total-isk-cost {
        font-weight: bold;
    }

    .panel-srp-request-overview-column {
        margin-bottom: 1rem;
    }

    .panel-srp-request-overview-cell {
        margin-bottom: 1rem;
    }

    .panel-srp-request-overview-cell:last-child {
        margin-bottom: 0;
    }

    .modal-srp-details {
        margin-bottom: 2rem;
    }

    .aasrp-character-portrait,
    .aasrp-evetype-icon {
        margin-right: 0.5rem;
    }

    .aasrp-character-portrait {
        vertical-align: baseline;
    }

    .aasrp-character-portrait-character-name {
        display: inline-block;
    }

    .aasrp-character-portrait-character-name small {
        font-size: 80%;
    }

    .aa-srp .table-vertical-middle td {
        vertical-align: middle;
    }

    .aa-srp .text-aasrp-form-description {
        font-weight: 400;
    }

    .aa-srp-fa-icon {
        cursor: pointer;
    }

    .aa-srp-fa-icon-right {
        margin-left: 1rem;
    }

    .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(0 136 204);
        cursor: pointer;
        display: block;
    }

    .srp-request-status-rejected .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(192 192 192);
        cursor: pointer;
        display: block;
    }

    .template-dark-mode .srp-request-status-rejected .srp-request-payout-amount-editable .srp-payout-amount {
        border-bottom: dashed 1px rgb(90 90 90);
        cursor: pointer;
        display: block;
    }

    .srp-request-payout-amount-editable .srp-payout-amount-changed {
        border-bottom: dashed 1px rgb(204 71 0) !important;
        cursor: pointer;
        display: block;
    }

    .modal-srp-details-request-history .request-history-comment {
        border-bottom: 1px solid rgb(229 229 229);
        margin-top: 1rem;
        padding-bottom: 1rem;
    }

    .template-dark-mode .modal-srp-details-request-history .request-history-comment {
        border-bottom: 1px solid rgb(70 69 69);
    }

    .modal-srp-details-request-history .request-history-comment:last-child {
        border-bottom: none;
        padding-bottom: initial;
    }

    .aa-srp-footer {
        border-top: 1px solid rgb(236 240 241);
        font-size: 85%;
        margin-top: 1rem;
        padding-top: 1rem;
    }

    .template-dark-mode .aa-srp-footer {
        border-color: rgb(70 69 69);
    }
}

@media all and (min-width: 992px) {
    .panel-srp-request-overview-column {
        margin-bottom: 0;
    }
}

/* DataTables
------------------------------------------------------------------------------------- */
@media all {
    .dataTables_wrapper > .form-inline {
        margin-bottom: 1rem;
    }

    .dataTables_wrapper > .form-inline select {
        margin-bottom: 1rem;
        margin-right: 1rem;
    }

    .dataTables_wrapper > .form-inline select:last-child {
        margin-right: 0;
    }

    .dataTables_length select {
        margin: 0 1rem;
    }
}

@media (min-width: 768px) {
    .dataTables_wrapper > .form-inline select:first-child {
        margin-left: 1rem;
    }
}

/* Editable Popup
------------------------------------------------------------------------------------- */
@media all {
    .editable-popup .arrow {
        display: none;
    }
}
